using Microsoft.Extensions.Caching.Distributed;
using Microsoft.Extensions.Caching.Memory;
using S01_ObjectCache.Services;

var builder = WebApplication.CreateBuilder(args);

// Register cache services (not needed when using MVC, Razor or Web API)
builder.Services.AddMemoryCache();
builder.Services.AddDistributedMemoryCache();

// Add Distributed SQL Server Cache
//builder.Services.AddDistributedSqlServerCache(options => {
//    options.ConnectionString = builder.Configuration.GetConnectionString("Cache");
//    options.SchemaName = "dbo";
//    options.TableName = "CacheStore";
//});

// Register the demo services
builder.Services.AddSingleton<TimeService>();
builder.Services.AddSingleton<TextFileService>();

// Show home page
var app = builder.Build();
app.MapGet("/", (HttpResponse rp) => {
    rp.ContentType = "text/html";
    return """
        <html>
            <head>
                <title>Object Cache</title>
            <head>
            <body>
                <h1>Object Cache</h1>
                <ul>
                    <li><a href="/nocache">No cache</a></li>
                    <li><a href="/object-cache-memory">Memory cache</a></li>
                    <li><a href="/object-cache-memory-dependency">Memory cache with file dependency</a></li>
                    <li><a href="/object-cache-distributed">Distributed cache</a></li>
                </ul>
            </body>
        </html>
        """;
});

// Object cache demos
app.MapGet("/nocache", (TimeService ts) => ts.GetCurrentTime());
app.MapGet("/object-cache-memory", (TimeService ts, IMemoryCache cache) => cache.GetOrCreate("CachedTimeValue", entry => {
    entry.SetAbsoluteExpiration(TimeSpan.FromSeconds(5));
    return ts.GetCurrentTime();
}));
app.MapGet("/object-cache-memory-dependency", (TextFileService txt) => txt.GetTextFileContent("App_Data/file.txt"));
app.MapGet("/object-cache-distributed", (TimeService ts, IDistributedCache cache) => {
    var dt = cache.GetString("CachedTimeValue");
    if (dt is null) {
        dt = ts.GetCurrentTime();
        cache.SetString("CachedTimeValue", dt, new DistributedCacheEntryOptions {
            AbsoluteExpirationRelativeToNow = TimeSpan.FromSeconds(5),
        });
    }
    return dt;
});


await app.RunAsync();
