using Microsoft.AspNetCore.OutputCaching;
using S03_OutputCache.Services;

var builder = WebApplication.CreateBuilder(args);

// Register the demo service
builder.Services.AddSingleton<TimeService>();

// Register output caching
builder.Services.AddOutputCache(options => {
    options.AddPolicy("Cache5", builder => {
        builder.Expire(TimeSpan.FromSeconds(5));
    });
    options.AddPolicy("Cache60", builder => {
        builder.Expire(TimeSpan.FromSeconds(60));
    });
    options.AddPolicy("Cache60X", builder => {
        builder.Expire(TimeSpan.FromSeconds(60));
        builder.SetVaryByQuery("x");
    });
    options.AddPolicy("Neigh", builder => {
        builder.Expire(TimeSpan.FromMinutes(15));
        builder.Tag("neigh");
    });
});

// Show home page
var app = builder.Build();
app.MapGet("/", (HttpResponse rp) => {
    rp.ContentType = "text/html";
    return """
        <html>
            <head>
                <title>Output Cache</title>
            <head>
            <body>
                <h1>Output Cache</h1>
                <ul>
                    <li><a href="/nocache">No cache</a></li>
                    <li><a href="/cache-5">Cache for 5 seconds</a></li>
                    <li><a href="/cache-60">Cache for 60 seconds, no vary by</a></li>
                    <li><a href="/cache-60x">Cache for 60 seconds, vary by x</a></li>
                    <li><a href="/neigh">Cache with tag</a></li>
                    <li><a href="/neigh-evict">Evict tagged</a></li>
                </ul>
            </body>
        </html>
        """;
});

// Output cache
app.UseOutputCache();
app.MapGet("/nocache", (TimeService ts) => ts.GetCurrentTime());
app.MapGet("/cache-5", (TimeService ts) => ts.GetCurrentTime()).CacheOutput("Cache5");
app.MapGet("/cache-60", (TimeService ts) => ts.GetCurrentTime()).CacheOutput("Cache60");
app.MapGet("/cache-60x", (TimeService ts) => ts.GetCurrentTime()).CacheOutput("Cache60x");
app.MapGet("/neigh", (TimeService ts) => ts.GetCurrentTime()).CacheOutput("Neigh");
app.MapGet("/neigh-evict", async (IOutputCacheStore cache, CancellationToken cancellationToken) => {
    await cache.EvictByTagAsync("neigh", cancellationToken);
    return "Evicted cache items tagged by `neigh`.";
});

await app.RunAsync();
